@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 24867 $
+ * $Revision: 27700 $
  * $Author: rjongbloed $
- * $Date: 2010-11-10 01:29:17 -0600 (Wed, 10 Nov 2010) $
+ * $Date: 2012-05-24 21:43:12 -0500 (Thu, 24 May 2012) $
  */
 
 
@@ -186,12 +186,12 @@ class PSound : public PBYTEArray
 
 
 /**
-   Abstract class for a generalised sound channel, 
-   and an implementation of PSoundChannel for old code that is not plugin-aware.
+   Abstract class for a generalised sound channel, and an implementation of
+   PSoundChannel for old code that is not plugin-aware.
    When instantiated, it selects the first plugin of the base class 
    "PSoundChannel"
 
-   As an abstract class, this represents a sound schannel. Drivers for real, 
+   As an abstract class, this represents a sound channel. Drivers for real, 
    platform dependent sound hardware will be ancestors of this class and 
    can be found in the plugins section of PTLib.
 
@@ -201,13 +201,12 @@ class PSound : public PBYTEArray
    both read and write audio data to once instance of a PSoundChannel
    class.
 
-   PSoundChanel instances are designed to be reentrant. The actual
+   PSoundChannel instances are designed to be reentrant. The actual
    usage model employed is left to the developer. One model could be
-   where one thread is responsible for 
-   construction, setup, opening and read/write operations. After creating and
-   eventually opening the channel this thread is responsible for
-   handling read/writes fast enough to avoid gaps in the generated  audio
-   stream.
+   where one thread is responsible for construction, setup, opening and 
+   read/write operations. After creating and eventually opening the channel
+   this thread is responsible for handling read/writes fast enough to avoid
+   gaps in the generated audio stream.
 
    Remaining operations may beinvoked from other threads.
    This includes Close() and actually gathering the necessary data to
@@ -238,7 +237,7 @@ class PSound : public PBYTEArray
    Similarly for reading, an entire buffer must be read before any of it is
    available to a Read() call. Note that once a buffer is filled you can read
    it a byte at a time if desired, but as soon as all the data in the buffer
-   is used returned, the next read will wait until the entire next buffer is
+   is used, the next read will wait until the entire next buffer is
    read from the hardware. So again, tailor the number and size of buffers to
    the application. To avoid being blocked until the buffer fills, you can use
    the StartRecording() function to initiate the buffer filling, and the
@@ -257,6 +256,7 @@ class PSoundChannel : public PChannel
   /**@name Construction */
   //@{
     enum Directions {
+      Closed = -1,
       Recorder,
       Player
     };
@@ -342,13 +342,9 @@ class PSoundChannel : public PChannel
 
     /**Get the name for the default sound devices/driver that is on this
        platform. Note that a named device may not necessarily do both
-       playing and recording so the arrays returned with the <code>dir</code>
+       playing and recording so the string returned with the <code>dir</code>
        parameter in each value is not necessarily the same.
 
-       This will return a list of uniqie device names across all of the available
-       drivers. If two drivers have identical names for devices, then the string
-       returned will be of the form driver+'\\t'+device.
-
        @return
        A platform dependent string for the sound player/recorder.
      */
@@ -361,6 +357,10 @@ class PSoundChannel : public PChannel
        playing and recording so the arrays returned with the <code>dir</code>
        parameter in each value is not necessarily the same.
 
+       This will return a list of unique device names across all of the available
+       drivers. If two drivers have identical names for devices, then the string
+       returned will be of the form driver+'\\t'+device.
+
        @return
        Platform dependent strings for the sound player/recorder.
      */
@@ -406,9 +406,23 @@ class PSoundChannel : public PChannel
     /// Get the name of the open channel
     virtual PString GetName() const;
 
+    /// Get the direction of the channel
+    Directions GetDirection() const
+    {
+      return activeDirection;
+    }
+
+    /// Get text representing the direction of the channel
+    static const char * GetDirectionText(Directions dir);
+
+    virtual const char * GetDirectionText() const
+    {
+      return GetDirectionText(activeDirection);
+    }
+
     /** Abort the background playing/recording of the sound channel.
-	There will be a logic assertion if you attempt to Abort a
-	sound channel operation, when the device is currently closed.
+        There will be a logic assertion if you attempt to Abort a
+        sound channel operation, when the device is currently closed.
 
        @return
        true if the sound has successfully been aborted.
@@ -499,55 +513,53 @@ class PSoundChannel : public PChannel
 
     /** Low level write (or play) to the channel. 
 
-	It will generate a logical assertion if you attempt write to a
-	channel set up for recording.
+        It will generate a logical assertion if you attempt write to a
+        channel set up for recording.
 
-       @param buf is a pointer to the data to be written to the
-       channel.  It is an error for this pointer to be NULL. A logical
-       assert will be generated when buf is NULL.
+        @param buf is a pointer to the data to be written to the
+        channel.  It is an error for this pointer to be NULL. A logical
+        assert will be generated when buf is NULL.
 
-       @param len Nr of bytes to send. If len equals the buffer size
+        @param len Nr of bytes to send. If len equals the buffer size
         set by SetBuffers() it will block for
         (1000*len)/(samplesize*samplerate) ms. Typically, the sample
         size is 2 bytes.  If len == 0, this will return immediately,
         where the return value is equal to the value of IsOpen().
  
-       @return true if len bytes were written to the channel,
-         otherwise false. The GetErrorCode() function should be 
+        @return true if len bytes were written to the channel,
+        otherwise false. The GetErrorCode() function should be 
         consulted after Write() returns false to determine what 
         caused the failure.
-
      */
     virtual PBoolean Write(const void * buf, PINDEX len);
 
 
     /** Low level write (or play) with watermark to the channel. 
 
-	It will generate a logical assertion if you attempt write to a
-	channel set up for recording.
+        It will generate a logical assertion if you attempt write to a
+        channel set up for recording.
 
-       @param buf is a pointer to the data to be written to the
-       channel.  It is an error for this pointer to be NULL. A logical
-       assert will be generated when buf is NULL.
+        @param buf is a pointer to the data to be written to the
+        channel.  It is an error for this pointer to be NULL. A logical
+        assert will be generated when buf is NULL.
 
-       @param len Nr of bytes to send. If len equals the buffer size
+        @param len Nr of bytes to send. If len equals the buffer size
         set by SetBuffers() it will block for
         (1000*len)/(samplesize*samplerate) ms. Typically, the sample
         size is 2 bytes.  If len == 0, this will return immediately,
         where the return value is equal to the value of IsOpen().
-	  
-	   @param mark Unique identifer to identify the write
+      
+        @param mark Unique identifer to identify the write
  
-       @return PTrue if len bytes were written to the channel,
-         otherwise PFalse. The GetErrorCode() function should be 
+        @return PTrue if len bytes were written to the channel,
+        otherwise PFalse. The GetErrorCode() function should be 
         consulted after Write() returns PFalse to determine what 
         caused the failure.
-
      */
     virtual PBoolean Write(
       const void * buf,         ///< Pointer to a block of memory to write.
       PINDEX len,               ///< Number of bytes to write.
-	  const void * mark         ///< Unique Marker to identify write
+      const void * mark         ///< Unique Marker to identify write
     );
 
     /** Get number of bytes written in last Write() operation. */
@@ -619,18 +631,18 @@ class PSoundChannel : public PChannel
        reached. The GetLastReadCount() function returns the actual number
        of bytes read.
 
-	It will generate a logical assertion if you attempt to read
-	from a PSoundChannel that is setup for playing.
+       It will generate a logical assertion if you attempt to read
+       from a PSoundChannel that is setup for playing.
 
        The GetErrorCode() function should be consulted after Read() returns
        false to determine what caused the failure.
 
-        @param len Nr of bytes to endeaveour to read from the sound
-        device. If len equals the buffer size set by SetBuffers() it
-        will block for (1000*len)/(samplesize*samplerate)
-        ms. Typically, the sample size is 2 bytes.  If len == 0, this
-        will return immediately, where the return value is equal to
-        the value of IsOpen().
+       @param len Nr of bytes to endeaveour to read from the sound
+       device. If len equals the buffer size set by SetBuffers() it
+       will block for (1000*len)/(samplesize*samplerate)
+       ms. Typically, the sample size is 2 bytes.  If len == 0, this
+       will return immediately, where the return value is equal to
+       the value of IsOpen().
 
        @param buf is a pointer to the empty data area, which will
        contain the data collected from the sound device.  It is an
@@ -735,7 +747,8 @@ class PSoundChannel : public PChannel
   //@}
 
   protected:
-    PSoundChannel * baseChannel;
+    PSoundChannel * m_baseChannel;
+    PReadWriteMutex m_baseMutex;
 
     /**This is the direction that this sound channel is opened for use
        in.  Should the user attempt to used this opened class instance
