@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 25645 $
+ * $Revision: 28039 $
  * $Author: rjongbloed $
- * $Date: 2011-05-01 19:19:49 -0500 (Sun, 01 May 2011) $
+ * $Date: 2012-07-16 01:17:33 -0500 (Mon, 16 Jul 2012) $
  */
 
 #include <ptlib.h>
@@ -47,6 +47,8 @@
   #pragma comment(lib, "mpr.lib")
 #endif
 
+#include <ptlib/msos/ptlib/ipsock.h>
+
 #if defined(P_WIN_COM) 
   #include <objbase.h>
   #ifdef _MSC_VER
@@ -92,7 +94,6 @@ void PTime::SetFromFileTime(const FILETIME & timestamp)
   ULARGE_INTEGER i;
   i.HighPart = timestamp.dwHighDateTime;
   i.LowPart = timestamp.dwLowDateTime;
-  i.QuadPart;
 
   theTime = (time_t)(i.QuadPart/scale - delta);
   microseconds = (long)(i.QuadPart%scale/10);
@@ -593,7 +594,6 @@ PBoolean PChannel::ConvertOSError(int status, Errors & lastError, int & osError)
     switch (osError) {
       case ERROR_INVALID_HANDLE :
       case WSAEBADF :
-      case WSAENOTSOCK :
         osError = EBADF;
         break;
       case ERROR_INVALID_PARAMETER :
@@ -610,15 +610,63 @@ PBoolean PChannel::ConvertOSError(int status, Errors & lastError, int & osError)
       case WSAEINTR :
         osError = EINTR;
         break;
+      case WSAEINPROGRESS :
+        osError = EINPROGRESS;
+        break;
+      case WSAENOTSOCK :
+        osError = ENOTSOCK;
+        break;
+      case WSAEOPNOTSUPP :
+        osError = EOPNOTSUPP;
+        break;
+      case WSAEAFNOSUPPORT :
+        osError = EAFNOSUPPORT;
+        break;
+      case WSAEADDRINUSE :
+        osError = EADDRINUSE;
+        break;
+      case WSAEADDRNOTAVAIL :
+        osError = EADDRNOTAVAIL;
+        break;
+      case WSAENETDOWN :
+        osError = ENETDOWN;
+        break;
+      case WSAENETUNREACH :
+        osError = ENETUNREACH;
+        break;
+      case WSAENETRESET :
+        osError = ENETRESET;
+        break;
+      case WSAECONNABORTED :
+        osError = ECONNABORTED;
+        break;
+      case WSAECONNRESET :
+        osError = ECONNRESET;
+        break;
+      case WSAENOBUFS :
+        osError = ENOBUFS;
+        break;
+      case WSAEISCONN :
+        osError = EISCONN;
+        break;
+      case WSAENOTCONN :
+        osError = ENOTCONN;
+        break;
+      case WSAECONNREFUSED :
+        osError = ECONNREFUSED;
+        break;
+      case WSAEHOSTUNREACH :
+        osError = EHOSTUNREACH;
+        break;
       case WSAEMSGSIZE :
-        osError |= PWIN32ErrorFlag;
-        lastError = BufferTooSmall;
-        return PFalse;
+        osError = EMSGSIZE;
+        break;
       case WSAEWOULDBLOCK :
+        osError = EWOULDBLOCK;
+        break;
       case WSAETIMEDOUT :
-        osError |= PWIN32ErrorFlag;
-        lastError = Timeout;
-        return PFalse;
+        osError = ETIMEDOUT;
+        break;
       default :
         osError |= PWIN32ErrorFlag;
     }
@@ -650,8 +698,13 @@ PBoolean PChannel::ConvertOSError(int status, Errors & lastError, int & osError)
       lastError = NotOpen;
       break;
     case EAGAIN :
+    case ETIMEDOUT :
+    case EWOULDBLOCK :
       lastError = Timeout;
       break;
+    case EMSGSIZE :
+      lastError = BufferTooSmall;
+      break;
     case EINTR :
       lastError = Interrupted;
       break;
@@ -709,7 +762,7 @@ UINT __stdcall PThread::MainFunction(void * threadPtr)
 */
 
   process.m_activeThreadMutex.Wait();
-  process.m_activeThreads[thread->threadId] = thread;
+  process.m_activeThreads[thread->m_threadId] = thread;
   process.m_activeThreadMutex.Signal();
 
   process.SignalTimerChange();
@@ -738,30 +791,28 @@ void PThread::Win32AttachThreadInput()
 {
 #ifndef _WIN32_WCE
   PProcess & process = PProcess::Current();
-  ::AttachThreadInput(threadId, ((PThread&)process).threadId, PTrue);
-  ::AttachThreadInput(((PThread&)process).threadId, threadId, PTrue);
+  ::AttachThreadInput(m_threadId, ((PThread&)process).m_threadId, PTrue);
+  ::AttachThreadInput(((PThread&)process).m_threadId, m_threadId, PTrue);
 #endif
 }
 
 
-PThread::PThread()
-  : autoDelete(false)
-  , m_isProcess(true)
-  , originalStackSize(0)
+PThread::PThread(bool isProcess)
+  : m_isProcess(isProcess)
+  , m_autoDelete(!isProcess)
+  , m_originalStackSize(0)
   , threadHandle(GetCurrentThread())
-  , threadId(GetCurrentThreadId())
+  , m_threadId(GetCurrentThreadId())
 {
-  if (!PProcess::IsInitialised())
+  if (isProcess)
     return;
 
-  m_isProcess = false;
-  autoDelete = true;
   DuplicateHandle(GetCurrentProcess(), threadHandle, GetCurrentProcess(), &threadHandle, 0, 0, DUPLICATE_SAME_ACCESS);
 
   PProcess & process = PProcess::Current();
 
   process.m_activeThreadMutex.Wait();
-  process.m_activeThreads[threadId] = this;
+  process.m_activeThreads[m_threadId] = this;
   process.m_activeThreadMutex.Signal();
 
   process.deleteThreadMutex.Wait();
@@ -774,28 +825,25 @@ PThread::PThread(PINDEX stackSize,
                  AutoDeleteFlag deletion,
                  Priority priorityLevel,
                  const PString & name)
-  : threadName(name)
-  , m_isProcess(false)
+  : m_isProcess(false)
+  , m_autoDelete(deletion == AutoDeleteThread)
+  , m_originalStackSize(stackSize)
+  , m_threadName(name)
 {
   PAssert(stackSize > 0, PInvalidParameter);
-  originalStackSize = stackSize;
-
-  autoDelete = deletion == AutoDeleteThread;
 
 #ifndef _WIN32_WCE
-  threadHandle = (HANDLE)_beginthreadex(NULL, stackSize, MainFunction,
-                                        this, CREATE_SUSPENDED, &threadId);
+  threadHandle = (HANDLE)_beginthreadex(NULL, stackSize, MainFunction, this, CREATE_SUSPENDED, &m_threadId);
 #else
    threadHandle = CreateThread(NULL, stackSize, 
-                               (LPTHREAD_START_ROUTINE)MainFunction,
-                               this, CREATE_SUSPENDED, (LPDWORD) &threadId);
+                       (LPTHREAD_START_ROUTINE)MainFunction, this, CREATE_SUSPENDED, (LPDWORD) &m_threadId);
 #endif
 
   PAssertOS(threadHandle != NULL);
 
   SetPriority(priorityLevel);
 
-  if (autoDelete) {
+  if (IsAutoDelete()) {
     PProcess & process = PProcess::Current();
     process.deleteThreadMutex.Wait();
     process.autoDeleteThreads.Append(this);
@@ -847,7 +895,7 @@ void PThread::CleanUp()
 
   PProcess & process = PProcess::Current();
   process.m_activeThreadMutex.Wait();
-  process.m_activeThreads.erase(threadId);
+  process.m_activeThreads.erase(m_threadId);
   process.m_activeThreadMutex.Signal();
 
   if (!IsTerminated())
@@ -860,19 +908,17 @@ void PThread::CleanUp()
 
 void PThread::Restart()
 {
-  if (!PAssert(originalStackSize != 0, "Cannot restart process/external thread") ||
+  if (!PAssert(m_originalStackSize != 0, "Cannot restart process/external thread") ||
       !PAssert(IsTerminated(), "Cannot restart running thread"))
     return;
 
   CleanUp();
 
 #ifndef _WIN32_WCE
-  threadHandle = (HANDLE)_beginthreadex(NULL,
-                         originalStackSize, MainFunction, this, 0, &threadId);
+  threadHandle = (HANDLE)_beginthreadex(NULL, m_originalStackSize, MainFunction, this, 0, &m_threadId);
 #else
-   threadHandle = CreateThread(NULL, originalStackSize, 
-                (LPTHREAD_START_ROUTINE) MainFunction,
-                  this, 0, (LPDWORD) &threadId);
+   threadHandle = CreateThread(NULL, m_originalStackSize, 
+                                  (LPTHREAD_START_ROUTINE)MainFunction, this, 0, (LPDWORD)&m_threadId);
 #endif
   PAssertOS(threadHandle != NULL);
 }
@@ -962,16 +1008,17 @@ PBoolean PThread::IsSuspended() const
 
 void PThread::SetAutoDelete(AutoDeleteFlag deletion)
 {
-  PAssert(deletion != AutoDeleteThread || this != &PProcess::Current(), PLogicError);
-  if (autoDelete == (deletion != AutoDeleteThread))
+  PAssert(deletion != AutoDeleteThread || (!m_isProcess && this != &PProcess::Current()), PLogicError);
+  bool newAutoDelete = (deletion == AutoDeleteThread);
+  if (m_autoDelete == newAutoDelete)
     return;
 
-  autoDelete = deletion == AutoDeleteThread;
+  m_autoDelete = newAutoDelete;
 
   PProcess & process = PProcess::Current();
 
   process.deleteThreadMutex.Wait();
-  if (autoDelete)
+  if (m_autoDelete)
     process.autoDeleteThreads.Append(this);
   else {
     process.autoDeleteThreads.DisallowDeleteObjects();
@@ -1117,7 +1164,7 @@ void PProcess::HouseKeepingThread::Main()
 
 bool PProcess::SignalTimerChange()
 {
-  if (m_shuttingDown) 
+  if (!PAssert(IsInitialised(), PLogicError) || m_shuttingDown) 
     return false;
 
   deleteThreadMutex.Wait();
@@ -1375,9 +1422,9 @@ PBoolean PProcess::SetGroupName(const PString & groupname, PBoolean)
 }
 
 
-DWORD PProcess::GetProcessID() const
+PProcessIdentifier PProcess::GetCurrentProcessID()
 {
-  return GetCurrentProcessId();
+  return ::GetCurrentProcessId();
 }
 
 
